<?php
/*
Plugin Name: VK FullSite Installer
Description: 選んだデモサイトデータを数ステップでインポートして再現できるプラグインです。
Version: 0.3.2.0
Author: Vektor,Inc.
Author URI: https://vektor-inc.co.jp
Plugin URI: https://vk-fullsite-installer.com
License: GPLv2 or later
License URI: https://www.gnu.org/licenses/gpl-2.0.html
*/

// Exit if accessed directly.
defined( 'ABSPATH' ) || exit;

//// 定数の定義 ////
define( 'VK_FULLSITE_INSTALLER_PLUGIN_FILE_PATH', __FILE__ );
define( 'VK_FULLSITE_INSTALLER_PLUGIN_SLUG', 'vk-fullsite-installer' );

// sites.json の URL
define( 'SITES_JSON_API_URL', 'https://vk-fullsite-installer.com/wp-json/vkfsiw/v1/sites' );

// サイトデータのカウンター用 API の URL
define( 'SITES_COUNTER_API_URL', 'https://vk-fullsite-installer.com/wp-json/vkfsiw/v1/counter' );

// ライセンス認証用 API の URL
define( 'LICENSE_CHECK_API_URL', 'https://vk-fullsite-installer.com/wp-json/vkfsiw/v1/license' );

// Vektor Passport の購入 URL
define( 'PASSPORT_PURCHASE_URL', 'https://vws.vektor-inc.co.jp/product/vektor-passport-1y' );

// sites.json の license_type の種類
define( 'VK_FULLSITE_INSTALLER_LICENSE_TYPE_FREE', 'free' );
define( 'VK_FULLSITE_INSTALLER_LICENSE_TYPE_PASSPORT', 'passport' );
define( 'VK_FULLSITE_INSTALLER_LICENSE_TYPE_SITE', 'site' );
define( 'VK_FULLSITE_INSTALLER_LICENSE_TYPE_PASSPORT_AND_SITE', 'passport_and_site' );
$license_type_name_array = [
	VK_FULLSITE_INSTALLER_LICENSE_TYPE_FREE => '無料',
	VK_FULLSITE_INSTALLER_LICENSE_TYPE_PASSPORT => 'Vektor Passport',
	VK_FULLSITE_INSTALLER_LICENSE_TYPE_SITE => 'サイトライセンス',
	VK_FULLSITE_INSTALLER_LICENSE_TYPE_PASSPORT_AND_SITE => 'Vektor Passport + サイトライセンス',
];

// 言語の種類
$language_name_array = [
	'ja' => '日本語',
	'en' => '英語',
];

// 変更しないテーブルの配列
$skip_table_array = array(
	'wp_users',
	'wp_usermeta',
);

// メモリ制限を設定
ini_set( 'memory_limit', '512M' );

// バッファリング開始
add_action( 'init', 'vkfsi_buffering' );
function vkfsi_buffering() {
	ob_start();
}

/**
 * Composer autoload
 */
$autoload_path = plugin_dir_path( __FILE__ ) . 'vendor/autoload.php';
// vendor ディレクトリがない状態で誤配信された場合に Fatal Error にならないようにファイルの存在確認.
if ( file_exists( $autoload_path ) ) {
	require_once $autoload_path;
}

// Load Modules
require_once plugin_dir_path( __FILE__ ) . 'inc/load.php';

/**
 * Plugin Update Checker
 */
if ( class_exists( 'YahnisElsts\PluginUpdateChecker\v5\PucFactory' ) ) {
	$myUpdateChecker = YahnisElsts\PluginUpdateChecker\v5\PucFactory::buildUpdateChecker( // phpcs:ignore.
		'https://license.vektor-inc.co.jp/check/?action=get_metadata&slug=vk-fullsite-installer',
		__FILE__,
		'vk-fullsite-installer'
	);
}

// 初回起動時にリダイレクト
register_activation_hook( __FILE__, 'vkfsi_activate' );
add_action( 'admin_init', 'vkfsi_redirect_to_import_page' );

function vkfsi_activate() {
	add_option( 'vkfsi_do_activation_redirect', true );
}

function vkfsi_redirect_to_import_page() {
	if ( get_option( 'vkfsi_do_activation_redirect', false ) ) {
		delete_option( 'vkfsi_do_activation_redirect' );
		if ( ! isset( $_GET[ 'activate-multi' ] ) && ! isset( $_GET[ 'imported' ] ) ) {
			wp_redirect( admin_url( 'options-general.php?page=vk-fullsite-installer' ) );
			exit;
		}
	}
}

// 管理画面にメニューを追加
add_action( 'admin_menu', 'vkfsi_add_admin_menu' );
function vkfsi_add_admin_menu() {
	add_options_page( 'VK FullSite Installer 設定', 'VK FullSite Installer', 'manage_options', 'vk-fullsite-installer', 'vkfsi_import_page' );
}

// SVGスプライトの出力
function vkfsi_output_svg_sprite() {
	$svg_path = plugin_dir_path( __FILE__ ) . 'assets/images/icon.svg';
	if ( file_exists( $svg_path ) ) {
		echo file_get_contents( $svg_path );
	}
}
add_action( 'admin_footer', 'vkfsi_output_svg_sprite' );

// 管理画面で CSS を読み込む
add_action( 'admin_enqueue_scripts', 'vkfsi_enqueue_admin_styles' );
function vkfsi_enqueue_admin_styles( $hook ) {
    if ( $hook != 'settings_page_vk-fullsite-installer' ) {
        return;
    }
    wp_enqueue_style( 'vkfsi_admin_css', plugin_dir_url( __FILE__ ) . 'assets/css/style.css' );
}

/**
 * インポート処理のメインメソッド
 */
function vkfsi_import_page() {

	// 処理終了メッセージの表示
	if ( isset( $_GET[ 'imported' ] ) ) {
		vkfsi_import_end();
		return;
	}

	// インポート処理
	if ( isset( $_POST[ 'start_import' ] ) ) {
		vkfsi_import_site();
		return;
	}

	// インポート用フォーム（サイト名等の入力画面）
	if ( isset( $_POST['select_site'] ) ) {
		vkfsi_display_import_form();
		return;
	}

	// サイトのリストを表示
	vkfsi_display_sites();
}


function vkfst_string_to_number( $number_string ){
	$number = (int) str_replace( ',', '', $number_string );
	return $number;
}

/**
 * 販売価格情報
 *
 * @param array $site
 *
 * @return array $price_data
 */
function vkfst_get_display_price_data( $site ) {
	$price_normal   = isset( $site['shop-item']['price'] ) ? $site['shop-item']['price'] : '';
	$price_discount = isset( $site['shop-item']['price_discount'] ) ? $site['shop-item']['price_discount'] : '';

	$price_data = array(
		'display_mode'   => '',
		'price_deleted'  => '',
		'price_discount' => '',
		'price_normal'   => '',
	);

	// 無料（ 0円入力のある場合は特別に0円になっている ）
	if ( empty( $price_normal ) && empty( $price_discount ) && '0' !== $price_discount ) {
		$price_data['display_mode'] = 'free';
	} elseif ( ! empty( $price_discount ) ) {
		$price_data['display_mode']   = 'discount';
		$price_data['price_deleted']  = number_format( vkfst_string_to_number( $price_normal ) );
		$price_data['price_discount'] = number_format( vkfst_string_to_number( $price_discount ) );
	} elseif ( '0' === $price_discount ) {
		$price_data['display_mode']   = 'discount_free';
		// 割引で 0 円にしているが通常価格が未記入の場合
		if ( $price_normal ){
			$price_data['price_deleted']  = number_format( vkfst_string_to_number( $price_normal ) );
		}
		$price_data['price_discount']  = number_format( vkfst_string_to_number( $price_discount ) );
	} elseif ( empty( $price_discount ) && ! empty( $price_normal ) ) {
		$price_data['display_mode']   = 'normal';
		$price_data['price_normal']  = number_format( vkfst_string_to_number( $price_normal ) );
	} else {
		$price_data['display_mode'] = 'normal';
		$price_data['price_normal'] = number_format( vkfst_string_to_number( $price_normal ) );
	}

	return $price_data;
}

function vkfst_get_display_price_html( $price_data ) {
	$price_html = '';

	// 打ち消し価格
	if ( ! empty( $price_data['price_deleted'] ) ){
		$price_html .= '<span class="vkfsi_price_del">¥' . number_format( vkfst_string_to_number( $price_data['price_deleted'] ) ) . '円<span class="vkfsi_price_tax">(税込)</span></span>';
	}

	// 割引価格
	if ( isset( $price_data['price_discount'] ) && '' !== $price_data['price_discount'] ){
		$price_html .= '<span class="vkfsi_price vkfsi_price_discount">¥<span class="vkfsi_price_number">' . number_format( vkfst_string_to_number( $price_data['price_discount'] ) ) . '</span>円<span class="vkfsi_price_tax">(税込)</span></span>';
	}

	// 通常価格
	if ( isset( $price_data['price_normal'] ) && '' !== $price_data['price_normal'] ){
		$price_html .= '<span class="vkfsi_price vkfsi_price_no_discount">¥<span class="vkfsi_price_number">' . number_format( vkfst_string_to_number(  $price_data['price_normal'] ) ) . '</span>円<span class="vkfsi_price_tax">(税込)</span></span>';
	}

	// 無料
	if ( isset( $price_data['display_mode'] ) && 'free' === $price_data['display_mode'] ){
		$price_html .= '<span class="vkfsi_price vkfsi_price_no_discount">無料</span>';
	}

	return $price_html;
}

/**
 * サイトのリストを表示
 */
function vkfsi_display_sites() {

	// API から sites.json を取得
	$sites_json = file_get_contents( SITES_JSON_API_URL );

	// JSON デコード
	$sites = json_decode( $sites_json, true );
	if ( json_last_error() !== JSON_ERROR_NONE ) {
		echo '<div class="notice notice-error is-dismissible"><p>sites.json ファイルの読み込みに失敗しました。</p></div>';
		return;
	}

	// sites.json ファイルの内容をフィルタリング
	$sites = apply_filters( 'vkfsi_sites', $sites );

	// 検索フォーム用に各値を取得
	$search_language_array = []; // 言語の配列
	$search_theme_array = []; // テーマ名の配列
	$search_theme_type_array = []; // テーマタイプの配列
	$search_license_type_array = []; // ライセンス区分の配列
	$search_author_array = []; // Author の配列
	foreach ( $sites as $site ) {
		$search_theme_array[] = $site[ 'theme' ];
		$search_theme_type_array[] = $site[ 'theme_type' ];
		$search_language_array[] = $site[ 'language' ];
		$search_license_type_array[] = $site[ 'license_type' ];
		$search_author_array[] = $site[ 'author' ];
	}

	// array_unique で重複を削除
	$search_theme_array = array_unique( $search_theme_array );
	$search_theme_type_array = array_unique( $search_theme_type_array );
	$search_language_array = array_unique( $search_language_array );
	$search_license_type_array = array_unique( $search_license_type_array );
	$search_author_array = array_unique( $search_author_array );

	// 入力されたライセンスキーを格納する変数
	// ただし、認証が失敗した場合は空文字にセットする
	$license_key_passport = '';
	$license_key_site = '';

	// 処理対象のサイトコード
	$site_code = '';

	// データダウンロード URL
	// 認証が通れば、認証サーバーが返してくる
	$data_url = '';

	// Vektor Passport ライセンスキーとサイトライセンスキー用のエラーフラグ
	// true なら入力値に問題があるので、エラーメッセージを表示する
	$error_flag_passport = false;
	$error_flag_site = false;

	// 「保存」ボタンが押されていれば、ライセンス認証を行う
	if ( isset( $_POST[ 'save_license_key_vektor_passport' ] ) || isset( $_POST[ 'save_license_key_site' ] ) ) {
		// サイトコードの取得
		$site_code = sanitize_text_field( $_POST[ 'vkfsi_code' ] );

		// Vektor Passport ライセンスキーの取得
		$license_key_passport = '';
		if ( isset( $_POST[ 'license_key_vektor_passport' ] ) ) {
			$license_key_passport = sanitize_text_field( $_POST[ 'license_key_vektor_passport' ] );
		}

		// サイトライセンスキーの取得
		$license_key_site = '';
		if ( isset( $_POST[ 'license_key_site' ] ) ) {
			$license_key_site = sanitize_text_field( $_POST[ 'license_key_site' ] );
		}

		// ライセンス認証 URL
		$license_check_url = LICENSE_CHECK_API_URL;
		$license_check_url = apply_filters( 'vkfsi_license_check_url', $license_check_url );

		// 認証クラスの初期化
		$license_checker = VK_FullSite_Installer_License_Checker::get_instance();
		$license_checker->set_api_url( $license_check_url );
		$license_checker->set_site_code( $site_code );
		$license_checker->set_passport_license_key( $license_key_passport );
		$license_checker->set_site_license_key( $license_key_site );

		// 認証処理
		$result = $license_checker->get_data();

		// 認証結果が返ってきた場合
		if ( $result ) {
			// 認証結果が失敗した場合
			if ( 'fail' == $result[ 'status' ] ) {
				// ライセンスキーは２つともクリアする
				$license_key_passport = '';
				$license_key_site = '';

			// サイトライセンスキーだけ認証成功の場合
			} else if ( 'success_site' == $result[ 'status' ] ) {
				// Vektor Passport ライセンスキーはクリアする
				$license_key_passport = '';

			// Vektor Passport ライセンスキーだけ認証成功の場合
			} else if ( 'success_passport' == $result[ 'status' ] ) {
				// サイトライセンスキーはクリアする
				$license_key_site = '';
			}

			// データダウンロード URL
			// 認証不可なら空文字が入ってくる
			$data_url = $result[ 'data_url' ];

		// 認証結果が返ってこない場合
		} else {
			// 入力値はクリアする
			$license_key_passport = '';
			$license_key_site = '';
			$data_url = '';
		}
	}

	// Vektor Passport ライセンスキーが認証エラーで空文字にされた場合
	// メッセージ通知用にエラーフラグを立てる
	if ( isset( $_POST[ 'license_key_vektor_passport' ] ) && ! empty( $_POST[ 'license_key_vektor_passport' ] ) ) {
		if ( '' == $license_key_passport ) {
			$error_flag_passport = true;
		}
	}

	// サイトライセンスキーが認証エラーで空文字にされた場合
	// メッセージ通知用にエラーフラグを立てる
	if ( isset( $_POST[ 'license_key_site' ]) && ! empty( $_POST[ 'license_key_site' ] ) ) {
		if ( '' == $license_key_site ) {
			$error_flag_site = true;
		}
	}

	// 指定された site_code があれば、そのサイトまでスクロールさせる
	$vkfsi_code = isset($_POST['vkfsi_code']) ? $_POST['vkfsi_code'] : '';
	?>
	<script>
		jQuery( function( $ ) {
			var site_code = '<?php echo esc_js( $vkfsi_code ); ?>';
			if ( site_code != '' ) {
				var target = $( '#div-' + site_code );
				if ( target.length ) {
					var position = target.offset().top;
					$( 'html, body' ).animate( { scrollTop: position }, 500 );
				}
			}
		});
	</script>
	<?php
	// サイト一覧画面ヘッダー
	echo '<div class="wrap vkfsi_admin-page">';
	echo '<h1 class="vkfsi_logo">';
	echo '<img src="' . plugin_dir_url( __FILE__ ) . 'assets/images/admin.svg" alt="VK FullSite Installer 設定" />';
	echo '</h1>';

	// 設定追加用のアクションフック
	do_action( 'vkfsi_add_settings' );

	// 検索フォーム
	echo '<div class="vkfsi_search-form">';
	echo '<form method="post" action="">';
	echo '<input type="hidden" name="s-search" value="on">';
	echo '<h3>サイト検索</h3>';
	echo '<div class="vkfsi_search-content">';

	// デフォルトの言語選択肢
	$default_language = '';
	if ( isset( $_POST[ 's-language' ] ) ) {
		$default_language = sanitize_text_field( $_POST[ 's-language' ] );
	} else {
		$locale = get_locale();
		if ( $locale != 'ja' ) {
			$locale = 'en';
		}
		$default_language = $locale;
	}

	// 検索フォーム - 言語
	// echo '<div class="vkfsi_search-item">';
	// echo '<strong>言語</strong>';
	// echo '<ul class="vkfsi_input-wrap">';
	// echo '<select name="s-language" id="s-language">';

	// global $language_name_array;
	// foreach ( $language_name_array as $language => $language_name ) {
	// 	$selected = '';
	// 	if ( $language == $default_language ) {
	// 		$selected = 'selected';
	// 	}
	// 	echo '<option value="' . esc_attr( $language ) . '" ' . $selected . '>';
	// 	echo esc_html( $language_name );
	// 	echo '</option>';
	// }
	// echo '</select>';
	// echo '</ul>';
	// echo '</div>';

	// 検索フォーム - テーマ
	echo '<div class="vkfsi_search-item">';
	echo '<strong>テーマ</strong>';
	echo '<ul class="vkfsi_input-wrap">';
	foreach ( $search_theme_array as $theme ) {
		$checked = '';
		if ( isset( $_POST[ 's-theme' ] ) && in_array( $theme, $_POST[ 's-theme' ] ) ) {
			$checked = 'checked';
		}
		echo '<li><label>';
		echo '<input type="checkbox" name="s-theme[]" value="' . esc_attr( $theme ) . '" ' . $checked . '>';
		echo esc_html( $theme );
		echo '</label></li>';
	}
	echo '</ul>';
	echo '</div>';

	// 検索フォーム - テーマタイプ
	echo '<div class="vkfsi_search-item">';
	echo '<strong>テーマタイプ</strong>';
	echo '<ul class="vkfsi_input-wrap">';
	echo '<select name="s-theme-type" id="s-theme-type">';
	echo '<option value="">指定なし</option>';
	foreach ( $search_theme_type_array as $theme_type ) {
		$selected = '';
		if ( isset( $_POST[ 's-theme-type' ] ) && $theme_type == $_POST[ 's-theme-type' ] ) {
			$selected = 'selected';
		}
		echo '<option value="' . esc_attr( $theme_type ) . '" ' . $selected . '>';
		echo esc_html( $theme_type );
		echo '</option>';
	}
	echo '</select>';
	echo '</ul>';
	echo '</div>';

	// 検索フォーム - ライセンス区分
	echo '<div class="vkfsi_search-item">';
	echo '<strong>ライセンス区分</strong>';
	echo '<ul class="vkfsi_input-wrap">';
	global $license_type_name_array;
	foreach ( $license_type_name_array as $license_type => $license_name ) {
		$checked = '';
		if ( isset( $_POST[ 's-license-type' ] ) && in_array( $license_type, $_POST[ 's-license-type' ] ) ) {
			$checked = 'checked';
		}
		echo '<li><label>';
		echo '<input type="checkbox" name="s-license-type[]" value="' . esc_attr( $license_type ) . '" ' . $checked . '>';
		echo esc_html( $license_name );
		echo '</label></li>';
	}
	echo '</ul>';
	echo '</div>';

	// 検索フォーム - Author
	echo '<div class="vkfsi_search-item">';
	echo '<strong>Author</strong>';
	echo '<ul class="vkfsi_input-wrap">';
	foreach ( $search_author_array as $author ) {
		$checked = '';
		if ( isset( $_POST[ 's-author' ] ) && in_array( $author, $_POST[ 's-author' ] ) ) {
			$checked = 'checked';
		}
		echo '<li><label>';
		echo '<input type="checkbox" name="s-author[]" value="' . esc_attr( $author ) . '" ' . $checked . '>';
		echo esc_html( $author );
		echo '</label></li>';
	}
	echo '</ul>';
	echo '</div>';

	// 検索フォーム - キーワード
	echo '<div class="vkfsi_search-item">';
	$keyword = '';
	if ( isset( $_POST[ 's-keyword' ] ) ) {
		$keyword = sanitize_text_field( $_POST[ 's-keyword' ] );
	}
	echo '<strong>キーワード</strong>';
	echo '<div class="vkfsi_input-wrap">';
	echo '<input type="text" name="s-keyword" value="' . $keyword . '">';
	echo '</div>';
	echo '</div>';
	echo '</div>'; // vkfsi_search-content

	// 検索フォーム - 検索ボタン
	echo '<input type="submit" value="検索" class="button button-primary">';

	echo '</form>';
	echo '</div>'; // vkfsi_search-form

	// 検索条件にマッチしたもののみを抽出
	$filtered_sites = [];
	foreach ( $sites as $site ) {
		// 検索条件にマッチしなければスキップ
		if ( ! vkfsi_search_filter( $site ) ) {
			continue;
		}
		$filtered_sites[] = $site;
	}

	if ( count( $filtered_sites ) == 0 ) {
		echo '<div class="notice notice-info is-dismissible"><p>該当するサイトが見つかりませんでした。</p></div>';
	} else {
		echo '<p>インストールするサイトを選択してください</p>';
	}

	// 検索条件用 hidden タグ
	$search_hidden = '';
	foreach ( $_POST as $key => $value ) {
		if ( 0 === strpos( $key, 's-' ) ) {
			if ( is_array( $value ) ) {
				foreach ( $value as $v ) {
					$v = sanitize_text_field( $v );
					$search_hidden .= '<input type="hidden" name="' . esc_attr( $key ) . '[]" value="' . esc_attr( $v ) . '">';
				}
			} else {
				$value = sanitize_text_field( $value );
				$search_hidden .= '<input type="hidden" name="' . esc_attr( $key ) . '" value="' . esc_attr( $value ) . '">';
			}
		}
	}

	// サイト一覧の表示
	echo '<div class="vkfsi_sites">';
	foreach ( $filtered_sites as $site ) {
		// サイトデータの表示
		echo '<div id="div-' . $site[ 'site_code' ] . '" class="vkfsi_site">';

		// フォーム
		echo '<form method="post" action="">';

		// サイトコードとライセンスタイプ
		echo '<input type="hidden" name="vkfsi_code" value="' . esc_html( $site[ 'site_code' ] ) . '">';
		echo '<input type="hidden" name="vkfsi_license_type" value="' . esc_html( $site[ 'license_type' ] ) . '">';

		// 検索条件用 hidden タグ
		echo $search_hidden;

		// サムネイル画像
		echo '<a href="' . esc_url( $site[ 'demo_url' ] ) . '" target="_blank">';
		echo '<img src="' . esc_url( $site[ 'thumbnail_url' ] ) . '" alt="' . esc_attr( $site[ 'site_name' ] ) . '" width="300" class="vkfsi_thumbnail">';
		echo '</a>';

		// サイト名の表示
		echo '<h3>' . esc_html( $site[ 'site_name' ] ) . '</h3>';

		$license_type = '';
		switch ( $site['license_type'] ) {
			case VK_FULLSITE_INSTALLER_LICENSE_TYPE_FREE:
				$license_type = '無料';
				break;
			case VK_FULLSITE_INSTALLER_LICENSE_TYPE_PASSPORT:
				$license_type = 'Vektor Passport';
				break;
			case VK_FULLSITE_INSTALLER_LICENSE_TYPE_PASSPORT_AND_SITE:
				$license_type = 'Vektor Passport + サイトライセンス';
				break;
			case VK_FULLSITE_INSTALLER_LICENSE_TYPE_SITE:
				$license_type = 'サイトライセンス';
				break;
			default:
				$license_type = '';
				break;
		}

		echo '<dl class="vkfsi_table"><dt><span class="vkfsi_table_label">ライセンスタイプ</span></dt><dd>' . $license_type . '</dd></dl>';
		// 使用テーマの表示
		echo '<dl class="vkfsi_table"><dt><span class="vkfsi_table_label">使用テーマ</span></dt><dd>' . $site[ 'theme' ] . '</dd></dl>';

		echo '<dl class="vkfsi_table"><dt><span class="vkfsi_table_label">テーマタイプ</span></dt><dd>' . $site[ 'theme_type' ] . '</dd></dl>';

		// 使用言語の表示
		// echo '<dl class="vkfsi_table"><dt><span class="vkfsi_table_label">使用言語</span></dt><dd>' . $site[ 'language' ] . '</dd></dl>';

		// Author の表示
		echo '<dl class="vkfsi_table"><dt><span class="vkfsi_table_label">Author</span></dt><dd>' . $site[ 'author' ] . '</dd></dl>';

		// Price ///////////////////////////////////////////////////////////////
		$price_data = vkfst_get_display_price_data( $site );
		$price_html = '<div class="vkfsi_price-outer">';
		$price_html .= vkfst_get_display_price_html( $price_data );
		if (
			VK_FULLSITE_INSTALLER_LICENSE_TYPE_PASSPORT_AND_SITE == $site[ 'license_type' ]
			|| VK_FULLSITE_INSTALLER_LICENSE_TYPE_PASSPORT == $site[ 'license_type' ]
			) {
			$price_html .= '<span class="vkfsi_price_passport">※ 別途 <a href="https://vws.vektor-inc.co.jp/vektor-passport" target="_blank">Vektor Passport</a> が必要です</span>';
		}
		$price_html .= '</div>'; // vkfsi_price-outer

		echo '<dl class="vkfsi_table"><dt><span class="vkfsi_table_label">販売価格</span></dt><dd>';
		echo $price_html;
		echo '</dd></dl>';

		echo '<div class="vkfsi_description">';
		echo esc_html( $site[ 'description' ] );
		echo '</div>';

		// $data_url はループの外からもってきているので、ループ内で $site の内容に応じて書き換えてしまうとるとインストール URL がおかしくなるので注意
		if ( VK_FULLSITE_INSTALLER_LICENSE_TYPE_FREE == $site[ 'license_type' ] ) {
			$submit_data_url = $site[ 'data_url' ];
		} else {
			$submit_data_url = $data_url;
		}
		echo '<input type="hidden" name="vkfsi_data_url" value="' . esc_url( $submit_data_url ) . '">';

		echo '<div class="vkfsi_btn-outer">';
		echo '<div class="vkfsi_btn-inner">';

		// デモサイトへのリンク表示
		echo '<p class="vkfsi_site-demo-url">
			<a href="' . esc_url( $site[ 'demo_url' ] ) . '" target="_blank" class="button vkfsi_button-with-icon">
				<span class="vkfsi_button-text">デモサイトを見る</span>
				<svg class="vkfsi_icon" width="18" height="18" aria-hidden="true">
					<use xlink:href="' . esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/icon.svg#icon-external-link' ) . '"></use>
				</svg>
			</a>
		</p>';

		// インポートボタンの表示処理
		// 無料版の場合は無条件で表示する
		if ( VK_FULLSITE_INSTALLER_LICENSE_TYPE_FREE == $site[ 'license_type' ] ) {
			echo '<p class="submit">
				<button type="submit" name="select_site" id="select_site" class="button button-primary vkfsi_button-with-icon">
					<span class="vkfsi_button-text">このサイトをインポート</span>
					<svg class="vkfsi_icon" width="18" height="18" aria-hidden="true">
						<use xlink:href="' . esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/icon.svg#icon-import' ) . '"></use>
					</svg>
				</button>
			</p>';

		// 有料版の場合は、ダウンロード URL が空文字でなければ、インポートボタンを表示
		} else {
			if ( '' != $data_url && $site_code == $site[ 'site_code' ] ) {
				echo '<p class="submit">
					<button type="submit" name="select_site" id="select_site" class="button button-primary vkfsi_button-with-icon">
						<span class="vkfsi_button-text">このサイトをインポート</span>
						<svg class="vkfsi_icon" width="18" height="18" aria-hidden="true">
							<use xlink:href="' . esc_url( plugin_dir_url( __FILE__ ) . 'assets/images/icon.svg#icon-import' ) . '"></use>
						</svg>
					</button>
				</p>';
			}
		}

		// Vektor Passport ライセンスキーの入力欄
		if ( VK_FULLSITE_INSTALLER_LICENSE_TYPE_PASSPORT_AND_SITE == $site[ 'license_type' ]
			|| VK_FULLSITE_INSTALLER_LICENSE_TYPE_PASSPORT == $site[ 'license_type' ] ) {

			echo '<label for="license_key_vektor_passport">Vektor Passport ライセンスキー</label>';
			echo '<div class="vkfsi_license-form">';

			// エラーメッセージを表示
			if ( $error_flag_passport && $site_code == $site[ 'site_code' ]) {
				echo '<div class="vkfsi_error">Vektor Passport ライセンスキーが間違っています。</div>';
			}

			// 保存ボタンを押したサイトコードと同じなら、
			// Vektor Passport ライセンスキーを表示する
			if ( $site_code == $site[ 'site_code' ] ) {
				echo '<input type="password" name="license_key_vektor_passport" value="' . $license_key_passport . '">';
			} else {
				echo '<input type="password" name="license_key_vektor_passport" value="">';
			}
			submit_button( '保存', 'primary', 'save_license_key_vektor_passport' );

			// 購入ボタン
			echo '<a href="' . PASSPORT_PURCHASE_URL . '" target="_blank">';
			echo '<button type="button" class="button button-primary">購入</button>';
			echo '</a>';
			echo '</div>';
		}

		// サイトライセンスキーの入力欄
		if ( VK_FULLSITE_INSTALLER_LICENSE_TYPE_PASSPORT_AND_SITE == $site[ 'license_type' ]
			|| VK_FULLSITE_INSTALLER_LICENSE_TYPE_SITE == $site[ 'license_type' ] ) {

			echo '<label for="license_key_site">サイト ライセンスキー</label>';
			echo '<div class="vkfsi_license-form">';

			// エラーメッセージを表示
			if ( $error_flag_site && $site_code == $site[ 'site_code' ] ) {
				echo '<div class="vkfsi_error">サイトライセンスキーが間違っています。</div>';
			}

			// 保存ボタンを押したサイトコードと同じなら、
			// サイトライセンスキーを表示する
			if ( $site_code == $site[ 'site_code' ] ) {
				echo '<input type="password" name="license_key_site" value="' . $license_key_site . '">';
			} else {
				echo '<input type="password" name="license_key_site" value="">';
			}
			submit_button( '保存', 'primary', 'save_license_key_site' );

			// 購入ボタン
			echo '<a href="' . $site['shop-item' ][ 'buy-link' ] . '" target="_blank">';
			echo '<button type="button" class="button button-primary">購入</button>';
			echo '</a>';
			echo '</div>';
		}

		echo '</div>';
		echo '</div>';
		echo '</form>';
		echo '</div>';
	}
	echo '</div>';
	echo '</div>';
}

/**
 * サイトデータをインポートするためのフォームを表示
 */
function vkfsi_display_import_form() {

	// パスポートライセンスキー
	$license_key_passport = '';
	if ( isset( $_POST[ 'license_key_vektor_passport' ] ) ) {
		$license_key_passport = sanitize_text_field( $_POST[ 'license_key_vektor_passport' ] );
	}

	// カレントユーザーの情報を取得
	$current_user = wp_get_current_user();

	// ユーザーのリストを取得
	$users = get_users( array(
		'fields' => array( 'ID', 'user_login' ),
	) );
	
	$user_select_form = '<select name="content_user_id">';
	foreach ( $users as $user ) {
		$selected = '';
		if ( $user->ID == $current_user->ID ) {
			$selected = 'selected';
		}
		$user_select_form .= '<option value="' . esc_attr( $user->ID ) . '" ' . $selected . '>';
		$user_select_form .= esc_html( $user->user_login );
		$user_select_form .= '</option>';
	}
	$user_select_form .= '</select>';

	?>
	<!-- ローディング画面 -->
	<div class="vkfsi_loading" style="display: none;">
		<p class="vkfsi_loading-text">Loading...</p>
		<div class="vkfsi_spinner"></div>
	</div>
	<!-- 入力チェック -->
	<script>
		jQuery( function( $ ) {
			$( '.vkfsi_start-import' ).on( 'click', function() {
				// 入力値のチェック
				if ( ! $( '#vkfsi_confirm_passport' ).prop( 'checked' ) ) {
					alert( 'チェックボックスをオンにしてください。' );
					return false;
				}

				$( '.vkfsi_loading' ).show();
			});
		});
	</script>

	<!-- インポートフォーム -->
	<div class="wrap vkfsi_admin-page">
		<h1 class="vkfsi_logo"><img src="<?php echo esc_url(plugin_dir_url(__FILE__) . 'assets/images/admin.svg'); ?>" alt="VK FullSite Installer 設定" /></h1>
		<h2>インポート設定</h2>
		<form method="post" action="">
			<input type="hidden" name="vkfsi_code" value="<?php echo esc_attr( $_POST[ 'vkfsi_code' ] ); ?>">
			<input type="hidden" name="vkfsi_data_url" value="<?php echo esc_url( $_POST[ 'vkfsi_data_url' ] ); ?>">
			<input type="hidden" name="vkfsi_license_key_vektor_passport" value="<?php echo esc_attr( $license_key_passport ); ?>">
			<table class="form-table">
				<tr>
					<th scope="row"><label for="content_user_id">コンテンツの所有者 <span class="description">(必須)</span></label></th>
					<td><?php echo $user_select_form; ?></td>
				</tr>
			</table>
			<p>
				<label>
					<input name="confirm_import" type="checkbox" id="vkfsi_confirm_passport" value="yes">
					<span class="text-danger">このセットアップは新規サイト構築専用です。インポートすると現在のサイトのデータは完全に消失します。</span>
				</label>
			</p>
			<?php submit_button( 'インポート開始', 'primary vkfsi_start-import', 'start_import' ); ?>
		</form>
	</div>
	<?php
}

/**
 * サイトデータをインポートする処理
 */
function vkfsi_import_site() {

	//// インポート処理前の入力値チェック ////

	// データ URL のチェック
	if ( ! isset( $_POST[ 'vkfsi_data_url' ] ) ) {
		echo '<div class="notice notice-error is-dismissible"><p>インポートするサイトを選択してください。</p></div>';
		return;
	}

	// 必須入力項目のチェック
	$validate_flag = true;

	if ( empty( $_POST[ 'content_user_id' ] ) ) {
		echo '<div class="notice notice-error is-dismissible"><p>コンテンツの所有者を指定してください。</p></div>';
		$validate_flag = false;
	}

	if ( ! isset( $_POST[ 'confirm_import' ] ) || $_POST[ 'confirm_import' ] !== 'yes' ) {
		echo '<div class="notice notice-error is-dismissible"><p>インポートを確認するチェックボックスをオンにしてください。</p></div>';
		$validate_flag = false;
	}

	// 入力エラーがあれば処理終了
	if ( ! $validate_flag ) {
		return;
	}

	//// ここからインポート処理 ////
	global $wpdb;

	// WordPress アドレスとサイトアドレスを事前に取得
	$site_url = get_site_url();
	$home_url = get_home_url();

	// WordPress のサイト名を取得
	$site_name = get_bloginfo( 'name' );

	// 各入力値の取得
	$data_url = esc_url_raw( $_POST[ 'vkfsi_data_url' ] );
	$content_user_id = intval( $_POST[ 'content_user_id' ] );

	// インポート用ディレクトリの作成
	$import_dir = WP_CONTENT_DIR . '/vk-fullsite-installer';
	if ( ! file_exists( $import_dir ) ) {
		mkdir( $import_dir, 0755, true );
	}

	// ZIP ファイルのダウンロード
	$zip_file = $import_dir . '/' . basename( $data_url );
	$response = wp_remote_get( $data_url, array( 'timeout' => 300 ) );
	if ( is_wp_error( $response ) ) {
		echo '<div class="notice notice-error is-dismissible"><p>インポートデータのダウンロードに失敗しました。</p></div>';
		return;
	}
	file_put_contents( $zip_file, wp_remote_retrieve_body( $response ) );
	unset( $response );

	// ZIP ファイルの解凍
	$zip = new ZipArchive();
	if ( $zip->open( $zip_file ) === TRUE ) {
		$zip->extractTo( $import_dir );
		$zip->close();
		unlink( $zip_file ); // ZIP ファイルを削除
	} else {
		echo '<div class="notice notice-error is-dismissible"><p>Zip ファイルの解凍に失敗しました。</p></div>';
		return;
	}
	unset( $zip );

	// Table Prefix の取得
	$prefix_file = $import_dir . '/prefix.txt';
	$table_prefix = 'wp_'; // デフォルトのプレフィックス
	if ( file_exists( $prefix_file ) ) {
		// プレフィックスファイルが存在する場合、プレフィックスを取得
		$table_prefix = file_get_contents( $prefix_file );
		unlink( $prefix_file ); // プレフィックスファイルを削除
	}

	// SQL ファイルのインポート
	$sql_file = $import_dir . '/site-export.sql';
	if ( file_exists( $sql_file ) ) {

		// wp_options テーブルのエクスポート部分から siteurl と home の値を取得
		$old_site_url = vkfsi_get_siteurl( $sql_file, $table_prefix );
		$old_home_url = vkfsi_get_home( $sql_file, $table_prefix );

		// 既存テーブルの DROP
		global $skip_table_array;
		$default_table_array = $wpdb->get_col( 'SHOW TABLES' );
		foreach ( $default_table_array as $table ) {
			// 指定プレフィックスのテーブルのみ DROP
			if ( strpos( $table, $wpdb->prefix ) !== 0 ) {
				continue;
			}

			// wp_users, wp_usermeta テーブルの DROP を除外
			$table_without_prefix = str_replace( $wpdb->prefix, 'wp_', $table );
			if ( in_array( $table_without_prefix, $skip_table_array ) ) {
				continue;
			}
			$table = str_replace( 'wp_', $wpdb->prefix, $table );
			$wpdb->query( 'DROP TABLE IF EXISTS `' . $table . '`' );
		}

		// 新規テーブルの作成
		vkfsi_create_tables( $sql_file, $table_prefix );

		// SQL ファイルを読み込みデータの INSERT を行う
		$handle = fopen( $sql_file, 'r' );
		if ( $handle ) {
			$create_table_flag = false;
			$insert_flag = false;
			$query = '';
			while ( ( $line = fgets( $handle ) ) !== false ) {
				if ( ! $create_table_flag && strpos( $line, 'CREATE TABLE `' . $table_prefix ) === 0 ) {
					$create_table_flag = true;
					continue;
				}
				if ( $create_table_flag && strpos( $line, ') ' ) === 0 ) {
					$create_table_flag = false;
					continue;
				}
				if ( $create_table_flag ) {
					continue;
				}

				// INSERT INTO の行をまとめる
				if ( strpos( $line, 'INSERT INTO ' ) === 0 ) {
					// INSERT 文が１行で終わる場合
					if ( strpos( $line, ";\n" ) !== false ) {
						// wp_users, wp_usermeta テーブルの INSERT 文を除外
						if ( strpos( $line, $table_prefix . 'users' ) !== false ) {
							continue;
						}
						if ( strpos( $line, $table_prefix . 'usermeta' ) !== false ) {
							continue;
						}
						$query = $line;
						$query = str_replace( 'INSERT INTO ' . $table_prefix, 'INSERT INTO ' . $wpdb->prefix, $query );
						$result = $wpdb->query( $query );
						$query = '';

					// INSERT 文が複数行にわたる場合
					} else {
						$insert_flag = true;
						$query = $line;
						continue;
					}
				}

				// INSERT 文が複数行にわたる場合
				if ( $insert_flag ) {
					// INSERT 文の終わりを検出
					if ( strpos( $line, ";\n" ) !== false ) {
						$query .= $line;

						// wp_users, wp_usermeta テーブルの INSERT 文を除外
						if ( strpos( $line, $table_prefix . 'users' ) === false
							 && strpos( $line, $table_prefix . 'usermeta' ) === false ) {
							$query = str_replace( 'INSERT INTO ' . $table_prefix, 'INSERT INTO ' . $wpdb->prefix, $query );
							$result = $wpdb->query( $query );
						}

						$query = '';
						$insert_flag = false;
					} else {
						$query .= $line;
					}
				}
			}
			fclose( $handle );
		}

		// Role のオプションを変更
		if ( $table_prefix != $wpdb->prefix ) {
			// wp_user_roles のオプション名を変更
			$query = 'UPDATE ' . $wpdb->prefix . 'options';
			$query .= ' SET option_name = "' . $wpdb->prefix . 'user_roles"';
			$query .= ' WHERE option_name = "' . $table_prefix . 'user_roles"';
			$result = $wpdb->query( $query );

			// wp_user_roles の値を取得して新しいオプションに追加
			$value = get_option( $wpdb->prefix . 'user_roles' );
			add_option( $table_prefix . 'user_roles', $value );
		}

		unlink( $sql_file ); // SQL ファイルを削除
	} else {
		echo '<div class="notice notice-error is-dismissible"><p>SQLファイルが見つかりません。</p></div>';
		return;
	}

	// wp_options テーブルの siteurl と home の値を更新
	if ( $old_site_url != '' && $old_home_url != '' ) {
		$wpdb->update( $wpdb->options, [ 'option_value' => $site_url ], [ 'option_name' => 'siteurl' ], [ '%s' ], '%s' );
		$wpdb->update( $wpdb->options, [ 'option_value' => $home_url ], [ 'option_name' => 'home' ], [ '%s' ], '%s' );
	}

	// テーブルの一覧を取得
	$tables = $wpdb->get_results( 'SHOW TABLES', ARRAY_N );
	foreach ( $tables as $table ) {
		$table_name = $table[ 0 ];
		if (strpos( $table_name, $wpdb->prefix ) !== 0) {
			continue; // プレフィックスが異なるテーブルはスキップ
		}
		vkfsi_replace_table_values( $table_name, $old_site_url, $site_url );
		vkfsi_replace_table_values( $table_name, $old_home_url, $home_url );
	}

	// wp-content 以下のファイルをインポート
	$content_dir = WP_CONTENT_DIR;
	$import_files = new RecursiveIteratorIterator(
		new RecursiveDirectoryIterator( $import_dir, RecursiveDirectoryIterator::SKIP_DOTS ),
		RecursiveIteratorIterator::LEAVES_ONLY
	);
	foreach ( $import_files as $file ) {
		if ( ! $file->isDir() ) {
			$file_path = $file->getRealPath();
			$relative_path = substr( $file_path, strlen( $import_dir ) + 1 );
			$target_path = $content_dir . '/' . $relative_path;

			if ( ! file_exists( dirname( $target_path ) ) ) {
				mkdir( dirname( $target_path ), 0755, true );
			}
			copy( $file_path, $target_path );
		}
	}

	// wp_posts の post_author を更新
	$result = $wpdb->query(
		$wpdb->prepare(
			"UPDATE {$wpdb->posts} SET post_author = %d",
			$content_user_id
		)
	);

	if ( false === $result ) {
		echo '<div class="notice notice-error is-dismissible"><p>記事の Author 情報の更新に失敗しました。</p></div>';
		return;
	}

	// サイト名を更新
	$result = update_option( 'blogname', '' );
	$result = update_option( 'blogname', $site_name );

	// キャッシュのクリア
	wp_cache_flush();

	// プラグインを有効化
	$activate_plugins = array( 'vk-fullsite-installer' );
	foreach ( $activate_plugins as $plugin ) {
		activate_plugins( $plugin . '/' . $plugin . '.php' );
	}

	// ZIP ファイル用のディレクトリを削除
	vkfsi_remove_directory( $import_dir );

	// Vektor Passport ライセンスキーの保存
	if ( ! empty( $_POST[ 'vkfsi_license_key_vektor_passport' ] ) ) {
		$license_key_passport = sanitize_text_field( $_POST[ 'vkfsi_license_key_vektor_passport' ] );

		// Lightning G3 Pro Unit のライセンスキーを保存
		update_option( 'lightning-g3-pro-unit-license-key', $license_key_passport );

		// VK AB Testing のライセンスキーを保存
		update_option( 'vk_ab_testing_license_key', $license_key_passport );

		// VK Blocks Pro のライセンスキーを保存
		$options = get_option( 'vk_blocks_options' );
		if ( ! is_array( $options ) ) {
			$options = array();
		}
		$options[ 'vk_blocks_pro_license_key' ] = $license_key_passport;
		update_option( 'vk_blocks_options', $options );
	}

	// サイトデータのカウントアップ
	$site_code = $_POST[ 'vkfsi_code' ];
	$api_url = SITES_COUNTER_API_URL . '?code=' . $site_code;
	$response = wp_remote_get( $api_url );

	// インポート後の処理へリダイレクト
	wp_redirect( admin_url( 'options-general.php?page=vk-fullsite-installer&imported=true' ) );
}

/**
 * インポート後の処理
 */
function vkfsi_import_end() {

	// パーマリンクの再設定
	flush_rewrite_rules();

	// 管理画面の "VK FullSite Installer 設定" を非表示にする
	?>
	<script>
		jQuery( function( $ ) {
			$( 'li.current' ).hide();
		});
	</script>
	<?php

	// インポート完了メッセージを表示
	echo '<div class="notice notice-success is-dismissible"><p>インポートが完了しました。</p></div>';

	// プラグインを無効化
	$deactivate_plugins = array( 'vk-fullsite-installer' );
	foreach ( $deactivate_plugins as $plugin ) {
		deactivate_plugins( $plugin . '/' . $plugin . '.php' );
	}
}

/**
 * wp_options テーブルのエクスポート部分から siteurl の値を取得
 * @param string $file SQL ファイルのパス
 * @param string $prefix テーブルのプレフィックス
 * @return string siteurl の値
 */
function vkfsi_get_siteurl( $file, $prefix ) {
	$value = '';

	$pattern = '/INSERT INTO ' . $prefix . 'options VALUES\(\'\d+\',\'siteurl\',\'(.*?)\'/';

	$handle = fopen( $file, 'r' );
	if ( $handle ) {
		while ( ( $line = fgets( $handle ) ) !== false ) {
			if ( preg_match( $pattern, $line, $matches ) ) {
				$value = $matches[1];
				break;
			}
		}
		fclose( $handle );
	}

	return $value;
}

/**
 * wp_options テーブルのエクスポート部分から home の値を取得
 * @param string $file SQL ファイルのパス
 * @param string $prefix テーブルのプレフィックス
 * @return string home の値
 */
function vkfsi_get_home( $file, $prefix ) {
	$value = '';

	$pattern = '/INSERT INTO ' . $prefix . 'options VALUES\(\'\d+\',\'home\',\'(.*?)\'/';

	$handle = fopen( $file, 'r' );
	if ( $handle ) {
		while ( ( $line = fgets( $handle ) ) !== false ) {
			if ( preg_match( $pattern, $line, $matches ) ) {
				$value = $matches[1];
				break;
			}
		}
		fclose( $handle );
	}

	return $value;
}

/**
 * 新規テーブルの作成
 * @param string $file SQL ファイルのパス
 */
function vkfsi_create_tables( $file, $prefix ) {
	global $wpdb;
	global $skip_table_array;

	// ファイルから CREATE TABLE の行を取得
	$handle = fopen( $file, 'r' );
	if ( $handle ) {
		$create_table_flag = false;
		$skip_flag = false;
		$query = '';
		while ( ( $line = fgets( $handle ) ) !== false ) {
			// デフォルトテーブルの CREATE TABLE の行をスキップ
			if ( preg_match( '/CREATE TABLE `' . $prefix . '([a-zA-Z0-9_]+)`/', $line, $matches ) ) {
				if ( in_array( $prefix . $matches[1], $skip_table_array ) ) {
					$skip_flag = true;
				} else {
					$skip_flag = false;
				}
				$create_table_flag = true;
			}

			if ( $skip_flag && strpos( $line, ') ' ) === 0 ) {
				$skip_flag = false;
				$create_table_flag = false;
				continue;
			}

			if ( ! $skip_flag && $create_table_flag ) {
				$query .= $line;

				if ( strpos( $line, ') ' ) === 0 ) {
					try {
						// テーブル名を置換してクエリを実行
						$query = str_replace( 'CREATE TABLE `' . $prefix, 'CREATE TABLE IF NOT EXISTS `' . $wpdb->prefix, $query );
						$wpdb->query( $query );
					} catch ( Exception $e ) {
						fclose( $handle );
						return;
					}
					$create_table_flag = false;
					$skip_flag = false;
					$query = '';
				}
			}
		}
		fclose( $handle );
	}
}

/**
 * 既存テーブルの値を置換
 * @param string $table_name テーブル名
 * @param string $old_url 置換前の URL
 * @param string $new_url 置換後の URL
 * @return void
 */
function vkfsi_replace_table_values( $table_name, $old_url, $new_url ) {
	global $wpdb;

	$rows = $wpdb->get_results( "SELECT * FROM $table_name", ARRAY_A );
	foreach ( $rows as $row ) {
		$replaced_row = array();
		foreach ( $row as $key => $value ) {
			$replaced_row[ $key ] = vkfsi_recursive_unserialize_replace( $old_url, $new_url, $value );
		}
		$wpdb->update( $table_name, $replaced_row, $row );
	}
}

/**
 * 再帰的にデータを置換
 * @param string $from 置換前の文字列
 * @param string $to 置換後の文字列
 * @param mixed $data 置換対象のデータ
 * @param bool $serialised データがシリアライズされているかどうか
 * @return mixed 置換後のデータ
 */
function vkfsi_recursive_unserialize_replace( $from, $to, $data, $serialised = false ) {

	try {
		if ( is_string( $data ) && ( $unserialized = @unserialize( $data ) ) !== false ) {
			$data = vkfsi_recursive_unserialize_replace( $from, $to, $unserialized, true );
		} elseif ( is_array( $data ) ) {
			$_tmp = array( );
			foreach ( $data as $key => $value ) {
				$_tmp[ $key ] = vkfsi_recursive_unserialize_replace( $from, $to, $value, false );
			}

			$data = $_tmp;
			unset( $_tmp );
		} else {
			if ( is_string( $data ) )
				$data = str_replace( $from, $to, $data );
        }

		if ( $serialised )
			return serialize( $data );

	} catch( Exception $error ) {
	}

	return $data;
}

/**
 * 再帰的にディレクトリを削除する
 * @param string $dir 削除するディレクトリのパス
 * @return bool 成功した場合は true、失敗した場合は false
 */
function vkfsi_remove_directory( $dir ) {
	// これより下位のディレクトリのみ対象とする
	$parent_dir = WP_CONTENT_DIR . '/vk-fullsite-installer/';

	// 親ディレクトリより下位のディレクトリのみ削除する
	if ( strpos( $dir, $parent_dir ) !== 0 && $dir != rtrim( $parent_dir, '/' ) ) {
		return;
	}

	$files = array_diff( scandir( $dir ), array( '.', '..' ) );
	foreach ($files as $file) {
		$path = $dir . '/' . $file;
		// ファイルかディレクトリかシンボリックリンクかによって処理を分ける
		if ( is_dir( $path ) ) {
			// ディレクトリなら再度同じ関数を呼び出す
			vkfsi_remove_directory( $path );
		} else if ( is_file( $path ) || is_link( $path ) ) {
			// ファイルまたはシンボリックリンクなら削除
			unlink( $path );
		}
	}
	// 指定したディレクトリを削除
	return rmdir( $dir );
}

/**
 * サイト一覧の表示フィルタリング（検索条件の適用）
 * @param array $site サイト情報
 * @return bool true: 検索条件にマッチする、false: 検索条件にマッチしない
 */
function vkfsi_search_filter( $site ) {
	if ( ! isset( $_POST[ 's-search' ] ) ) {
		return true;
	}

	// 言語
	if ( isset( $_POST[ 's-language' ] ) ) {
		if ( $site[ 'language' ] != $_POST[ 's-language' ] ) {
			return false;
		}
	}

	// テーマ
	if ( isset( $_POST[ 's-theme' ] ) ) {
		$array = $_POST[ 's-theme' ];
		if ( ! in_array( $site[ 'theme' ], $array ) ) {
			return false;
		}
	}

	// テーマタイプ
	if ( isset( $_POST[ 's-theme-type' ] ) && ! empty( $_POST[ 's-theme-type' ] ) ) {
		if ( $site[ 'theme_type' ] != $_POST[ 's-theme-type' ] ) {
			return false;
		}
	}

	// ライセンス区分
	if ( isset( $_POST[ 's-license-type' ] ) ) {
		$array = $_POST[ 's-license-type' ];
		if ( ! in_array( $site[ 'license_type' ], $array ) ) {
			return false;
		}
	}

	// Author
	if ( isset( $_POST[ 's-author' ] ) ) {
		$array = $_POST[ 's-author' ];
		if ( ! in_array( $site[ 'author' ], $array ) ) {
			return false;
		}
	}

	// キーワード
	if ( isset( $_POST[ 's-keyword' ] ) ) {
		$input_keyword = sanitize_text_field( $_POST[ 's-keyword' ] );
		$input_keyword = str_replace( '　', ' ', $input_keyword );
		$keyword_array = explode( ' ', $input_keyword );
		$match_counter = 0;
		foreach ( $keyword_array as $keyword ) {
			if ( strpos( $site[ 'site_name' ], $keyword ) !== false
				|| strpos( $site[ 'description' ], $keyword ) !== false ) {
				$match_counter++;
			}
		}
		if ( $match_counter != count( $keyword_array ) ) {
			return false;
		}
	}

	return true;
}

function vkfsi_enqueue_password_scripts( $hook_suffix ) {
	// 対象ページを判定
	if ( $hook_suffix !== 'settings_page_vk-fullsite-installer' ) {
		return;
	}

	// WordPress のパスワード系スクリプト（強度＆生成）
	wp_enqueue_script( 'user-profile' );

	// dashicons を使いたい場合
	wp_enqueue_style( 'dashicons' );
}
add_action( 'admin_enqueue_scripts', 'vkfsi_enqueue_password_scripts' );


/**
 * プラグイン一覧画面に「設定」リンクを追加
 * @param array $links プラグインのアクションリンク
 * @return array 更新されたアクションリンク
 */
function vkfsi_settings_link( $links ) {
	$settings_link = '<a href="options-general.php?page=vk-fullsite-installer">' . __( 'Settings', 'default' ) . '</a>';
	array_unshift( $links, $settings_link ); // 先頭に追加
	return $links;
}

add_filter( 'plugin_action_links_' . plugin_basename(__FILE__), 'vkfsi_settings_link' );
